@@ -11,5 +11,5 @@
     Mozharovskyi, P., Mosler, K. and Lange, T. (2013). Classifying real-world data with the DDalpha-procedure. Mimeo.
 */
 
-void GetDSpace(vector<TPoint> points, TVariables cardinalities, int k, bool atOnce, vector<TPoint> *dSpace, vector<TPoint> *directions, vector<TPoint> *projections);
-void GetDepths(TPoint point, vector<TPoint> points, TVariables cardinalities, int k, bool atOnce, vector<TPoint> directions, vector<TPoint> projections, TPoint *depths);
+void GetDSpace(vector<TPoint>& points, TVariables& cardinalities, int k, bool atOnce, vector<TPoint> *dSpace, vector<TPoint> *directions, vector<TPoint> *projections);
+void GetDepths(TPoint& point, vector<TPoint>& points, TVariables& cardinalities, int k, bool atOnce, vector<TPoint>& directions, vector<TPoint>& projections, TPoint *depths);
